use futures::join;
use glommio::{Latency, Local, LocalExecutorPoolBuilder, Placement, Shares};
use std::time::{Duration, Instant};

macro_rules! task_queue {
    ( $name:ident ) => {{
        let $name = Local::create_task_queue(
            Shares::Static(1000),
            Latency::Matters(Duration::from_millis(1)),
            stringify!($name),
        );

        let id = Local::id();
        Local::local_into(
            async move {
                let start = Instant::now();
                let mut count = 0;

                while start.elapsed() < Duration::from_secs(20) {
                    Local::yield_if_needed().await;
                    count += 1;
                }
                println!(
                    "executor {}, tq {} Ran {} times",
                    id,
                    stringify!($name),
                    count
                );
            },
            $name,
        )
        .unwrap()
    }};
}

fn main() {
    let handles = LocalExecutorPoolBuilder::new(32)
        .placement(Placement::MaxPack(None))
        .on_all_shards(|| async move {
            let t1 = task_queue!(tq1);
            let t2 = task_queue!(tq1);
            let t3 = task_queue!(tq1);
            let t4 = task_queue!(tq1);

            join!(t1, t2, t3, t4);
        })
        .unwrap();

    handles.join_all();
}
